@@ -528,13 +528,13 @@ def calculate_histogram_widget_data(display_data_and_metadata: typing.Optional[D
     display_data = display_data_and_metadata.data if display_data_and_metadata else None
     display_data_and_metadata = None  # release ref for gc. needed for tests, because this may occur on a thread.
     if display_data is not None:
-        total_pixels = numpy.product(display_data.shape, dtype=numpy.uint64)  # type: ignore
+        total_pixels = numpy.product(display_data.shape, dtype=numpy.uint64)
         if not subsample and subsample_fraction:
             subsample = min(max(total_pixels * subsample_fraction, subsample_min), total_pixels)
         if subsample:
-            data_sample = numpy.random.choice(display_data.reshape(numpy.product(display_data.shape, dtype=numpy.uint64)), subsample)  # type: ignore
+            data_sample = numpy.random.choice(display_data.reshape(numpy.product(display_data.shape, dtype=numpy.uint64)), subsample)
         else:
-            data_sample = numpy.copy(display_data)  # type: ignore
+            data_sample = numpy.copy(display_data)
         if display_range is None or data_sample is None:
             return HistogramWidgetData()
         # numpy is slow because it throws out data less/greater than the min/max values
@@ -550,7 +550,7 @@ def calculate_histogram_widget_data(display_data_and_metadata: typing.Optional[D
             histogram_data = numpy.zeros((bins,), dtype=int)
         # why can't numpy make this optimization?
         # histogram_data = factor * numpy.histogram(data_sample, range=display_range, bins=bins)[0]  # type: ignore
-        histogram_max = numpy.max(histogram_data)  # type: ignore  # assumes that histogram_data is int
+        histogram_max = numpy.max(histogram_data)  # assumes that histogram_data is int
         if histogram_max > 0:
             histogram_data = histogram_data / float(histogram_max)  # type: ignore
         return HistogramWidgetData(histogram_data, display_range)
