@@ -133,6 +133,14 @@ class ImageCanvasItemMapping(Graphics.CoordinateMappingLike):
     def calibrated_origin_widget(self) -> Geometry.FloatPoint:
         return self.map_point_image_to_widget(self.calibrated_origin_image)
 
+    def map_point_channel_norm_to_channel(self, x: float) -> float:
+        raise NotImplementedError()
+
+    def map_point_channel_norm_to_widget(self, x: float) -> float:
+        raise NotImplementedError()
+
+    def map_point_widget_to_channel_norm(self, pos: Geometry.FloatPoint) -> float:
+        raise NotImplementedError()
 
 
 class GraphicsCanvasItem(CanvasItem.AbstractCanvasItem):
@@ -223,11 +231,12 @@ class ScaleMarkerCanvasItem(CanvasItem.AbstractCanvasItem):
         dimensional_calibration = self.__dimensional_calibration
         if dimensional_calibration is not None:  # display scale marker?
             screen_pixel_per_image_pixel = self.__screen_pixel_per_image_pixel_stream.value
-            if screen_pixel_per_image_pixel and screen_pixel_per_image_pixel > 0.0:
+            dimensional_scale = dimensional_calibration.scale
+            if screen_pixel_per_image_pixel and screen_pixel_per_image_pixel > 0.0 and not math.isnan(dimensional_scale):
                 scale_marker_image_width = self.scale_marker_width / screen_pixel_per_image_pixel
-                calibrated_scale_marker_width = Geometry.make_pretty2(scale_marker_image_width * dimensional_calibration.scale, True)
+                calibrated_scale_marker_width = Geometry.make_pretty2(scale_marker_image_width * dimensional_scale, True)
                 # update the scale marker width
-                scale_marker_image_width = calibrated_scale_marker_width / dimensional_calibration.scale
+                scale_marker_image_width = calibrated_scale_marker_width / dimensional_scale
                 scale_marker_width = round(scale_marker_image_width * screen_pixel_per_image_pixel)
                 text1 = dimensional_calibration.convert_to_calibrated_size_str(scale_marker_image_width)
                 text2 = self.__info_text
@@ -593,7 +602,7 @@ class ImageCanvasItem(DisplayCanvasItem.DisplayCanvasItem):
                 # is a small percentage of the overall data and was falling outside
                 # the included range. This is the new simplified algorithm. Future
                 # feature may allow user to select more complex algorithms.
-                mn, mx = numpy.nanmin(data), numpy.nanmax(data)  # type: ignore
+                mn, mx = numpy.nanmin(data), numpy.nanmax(data)
                 delegate.update_display_data_channel_properties({"display_limits": (mn, mx)})
         return True
 
