@@ -287,16 +287,16 @@ class LineGraphAxes:
         y_calibration = self.y_calibration
         if y_calibration:
             if self.data_style == "log":
-                calibrated_data = y_calibration.offset + y_calibration.scale * uncalibrated_xdata.data  # type: ignore
-                calibrated_data[calibrated_data <= 0] = numpy.nan  # type: ignore
-                numpy.log10(calibrated_data, out=calibrated_data)  # type: ignore
+                calibrated_data = y_calibration.offset + y_calibration.scale * uncalibrated_xdata.data
+                calibrated_data[calibrated_data <= 0] = numpy.nan
+                numpy.log10(calibrated_data, out=calibrated_data)
             else:
-                calibrated_data = y_calibration.offset + y_calibration.scale * uncalibrated_xdata.data  # type: ignore
+                calibrated_data = y_calibration.offset + y_calibration.scale * uncalibrated_xdata.data
         else:
             if self.data_style == "log":
-                calibrated_data = uncalibrated_xdata.data.copy()  # type: ignore
-                calibrated_data[calibrated_data <= 0] = numpy.nan  # type: ignore
-                numpy.log10(calibrated_data, out=calibrated_data)  # type: ignore
+                calibrated_data = uncalibrated_xdata.data.copy()
+                calibrated_data[calibrated_data <= 0] = numpy.nan
+                numpy.log10(calibrated_data, out=calibrated_data)
             else:
                 calibrated_data = uncalibrated_xdata.data
         if calibrated_data is not None:
@@ -326,7 +326,7 @@ def are_axes_equal(axes1: typing.Optional[LineGraphAxes], axes2: typing.Optional
     return True
 
 
-def draw_background(drawing_context: DrawingContext.DrawingContext, plot_rect: Geometry.IntRect, background_color: typing.Optional[str]) -> None:
+def draw_background(drawing_context: DrawingContext.DrawingContext, plot_rect: Geometry.IntRect, background_color: typing.Optional[typing.Union[str, DrawingContext.LinearGradient]]) -> None:
     with drawing_context.saver():
         drawing_context.begin_path()
         drawing_context.rect(plot_rect.left, plot_rect.top, plot_rect.width, plot_rect.height)
