@@ -343,6 +343,10 @@ class ScriptListCanvasItemDelegate(Widgets.ListCanvasItemDelegate):
                                               rect[0][0] + 20 - 4)
 
 
+class ScriptCancelException(Exception):
+    pass
+
+
 class RunScriptDialog(Dialog.ActionDialog):
 
     def __init__(self, document_controller: "DocumentController.DocumentController"):
@@ -638,7 +642,7 @@ class RunScriptDialog(Dialog.ActionDialog):
                     ui_version = ui_version if ui_version else "~1.0"
                     api = PlugInManager.api_broker_fn(version, ui_version)
                     queued_api = FacadeQueued.API(api, None)  # type: ignore
-                    queued_api._queue_task = api.queue_task  # type: ignore
+                    queued_api._queue_task = api.queue_task
                     return queued_api
 
                 def get_ui(self, version: str) -> Declarative.DeclarativeUI:
@@ -667,7 +671,10 @@ class RunScriptDialog(Dialog.ActionDialog):
 
                 stdout = StdoutCatcher()
                 with contextlib.redirect_stdout(typing.cast(typing.Any, stdout)), contextlib.redirect_stderr(typing.cast(typing.Any, stdout)):
-                    exec(compiled, g)
+                    try:
+                        exec(compiled, g)
+                    except ScriptCancelException as e:
+                        self.print(_("Script canceled by user."))
             except Exception:
                 self.print("{}: {}".format(_("Error"), traceback.format_exc()))
 
@@ -756,14 +763,14 @@ class RunScriptDialog(Dialog.ActionDialog):
                 self.document_controller.add_task("ui_" + str(id(self)), self.__handle_output_and_q)
             accept_event.wait()
             if self.__is_closed:
-                raise Exception("Cancel")
+                raise ScriptCancelException()
 
         def update_message_column() -> None:
             self.__message_column.remove_all()
             self.__message_column.add(self.__make_cancel_row())
         self.document_controller.add_task("ui_" + str(id(self)), update_message_column)
         if result is None:
-            raise Exception("Cancel")
+            raise ScriptCancelException()
         return result
 
     def get_integer(self, prompt: str, default_value: int = 0) -> int:
@@ -793,7 +800,7 @@ class RunScriptDialog(Dialog.ActionDialog):
                 self.document_controller.add_task("show_" + str(id(self)), self.__handle_output_and_q)
             accept_event.wait()
             if self.__is_closed:
-                raise Exception("Cancel")
+                raise ScriptCancelException()
 
     def __register_sync_event(self, sync_event: threading.Event) -> None:
         self.__sync_events.add(sync_event)
@@ -833,7 +840,7 @@ class RunScriptDialog(Dialog.ActionDialog):
                 self.document_controller.add_task("ui_" + str(id(self)), self.__handle_output_and_q)
             accept_event.wait()
             if self.__is_closed:
-                raise Exception("Cancel")
+                raise ScriptCancelException()
 
         def update_message_column() -> None:
             self.__message_column.remove_all()
