@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 # standard libraries
 import functools
 import logging
@@ -7,6 +9,7 @@ import uuid
 import weakref
 
 # local libraries
+from nion.swift.model import Cache
 from nion.swift.model import Changes
 from nion.swift.model import Connection
 from nion.swift.model import DataGroup
@@ -19,13 +22,13 @@ from nion.swift.model import Persistence
 from nion.swift.model import WorkspaceLayout
 from nion.utils import Converter
 from nion.utils import ListModel
-from nion.utils import Observable
 
+PersistentDictType = typing.Dict[str, typing.Any]
 
 ProjectItemType = typing.Union[DataItem.DataItem, DisplayItem.DisplayItem, DataStructure.DataStructure, Connection.Connection, Symbolic.Computation]
 
 
-class Project(Observable.Observable, Persistence.PersistentObject):
+class Project(Persistence.PersistentObject):
     """A project manages raw data items, display items, computations, data structures, and connections.
 
     Projects are stored in project indexes, which are files that describe how to find data and and tracks the other
@@ -36,73 +39,138 @@ class Project(Observable.Observable, Persistence.PersistentObject):
 
     PROJECT_VERSION = 3
 
-    _processing_descriptions = dict()
+    _processing_descriptions: PersistentDictType = dict()
 
-    def __init__(self, storage_system: FileStorageSystem.ProjectStorageSystem):
+    def __init__(self, storage_system: FileStorageSystem.ProjectStorageSystem, cache_factory: typing.Optional[Cache.CacheFactory] = None) -> None:
         super().__init__()
 
         self.define_type("project")
-        self.define_property("title", str())
-        self.define_relationship("data_items", data_item_factory, insert=self.__data_item_inserted, remove=self.__data_item_removed)
-        self.define_relationship("display_items", display_item_factory, insert=self.__display_item_inserted, remove=self.__display_item_removed)
-        self.define_relationship("computations", computation_factory, insert=self.__computation_inserted, remove=self.__computation_removed)
-        self.define_relationship("data_structures", data_structure_factory, insert=self.__data_structure_inserted, remove=self.__data_structure_removed)
-        self.define_relationship("connections", Connection.connection_factory, insert=self.__connection_inserted, remove=self.__connection_removed)
-        self.define_relationship("data_groups", DataGroup.data_group_factory, insert=self.__data_group_inserted, remove=self.__data_group_removed)
-        self.define_relationship("workspaces", WorkspaceLayout.factory)
-        self.define_property("workspace_uuid", converter=Converter.UuidToStringConverter())
-        self.define_property("data_item_references", dict(), hidden=True, changed=self.__property_changed)  # map string key to data item, used for data acquisition channels
-        self.define_property("mapped_items", list(), changed=self.__property_changed)  # list of item references, used for shortcut variables in scripts
+        self.define_property("title", str(), hidden=True)
+        self.define_relationship("data_items", data_item_factory, insert=self.__data_item_inserted, remove=self.__data_item_removed, hidden=True)
+        self.define_relationship("display_items", display_item_factory, insert=self.__display_item_inserted, remove=self.__display_item_removed, hidden=True)
+        self.define_relationship("computations", computation_factory, insert=self.__computation_inserted, remove=self.__computation_removed, hidden=True)
+        self.define_relationship("data_structures", data_structure_factory, insert=self.__data_structure_inserted, remove=self.__data_structure_removed, hidden=True)
+        self.define_relationship("connections", Connection.connection_factory, insert=self.__connection_inserted, remove=self.__connection_removed, hidden=True)
+        self.define_relationship("data_groups", DataGroup.data_group_factory, insert=self.__data_group_inserted, remove=self.__data_group_removed, hidden=True)
+        self.define_relationship("workspaces", WorkspaceLayout.factory, hidden=True)
+        self.define_property("workspace_uuid", converter=Converter.UuidToStringConverter(), hidden=True)
+        self.define_property("data_item_references", dict(), changed=self.__property_changed, hidden=True)  # map string key to data item, used for data acquisition channels
+        self.define_property("mapped_items", list(), changed=self.__property_changed, hidden=True)  # list of item references, used for shortcut variables in scripts
 
-        self.handle_start_read = None
-        self.handle_insert_model_item = None
-        self.handle_remove_model_item = None
-        self.handle_finish_read = None
+        self.handle_start_read: typing.Optional[typing.Callable[[], None]] = None
+        self.handle_insert_model_item: typing.Optional[typing.Callable[[Persistence.PersistentContainerType, str, int, Persistence.PersistentObject], None]] = None
+        self.handle_remove_model_item: typing.Optional[typing.Callable[[Persistence.PersistentContainerType, str, Persistence.PersistentObject, bool], Changes.UndeleteLog]] = None
+        self.handle_finish_read: typing.Optional[typing.Callable[[], None]] = None
 
         self.__has_been_read = False
 
-        self._raw_properties = None  # debugging
+        self._raw_properties: typing.Optional[PersistentDictType] = None
+        self.__reader_errors: typing.Sequence[FileStorageSystem.ReaderError] = list()
 
         self.__storage_system = storage_system
 
         self.set_storage_system(self.__storage_system)
 
+        self.__cache_factory = cache_factory
+        self.__cache = cache_factory.create_cache() if cache_factory else None
+
     def close(self) -> None:
         self.handle_start_read = None
         self.handle_insert_model_item = None
         self.handle_remove_model_item = None
         self.handle_finish_read = None
+        if self.__cache_factory:
+            self.__cache_factory.release_cache(typing.cast(Cache.CacheLike, self.__cache))
+            self.__cache_factory = None
+            self.__cache = None
         self.__storage_system.close()
-        self.__storage_system = None
+        self.__storage_system = typing.cast(typing.Any, None)
         super().close()
 
+    @property
+    def title(self) -> str:
+        return typing.cast(str, self._get_persistent_property_value("title"))
+
+    @title.setter
+    def title(self, value: str) -> None:
+        self._set_persistent_property_value("title", value)
+
+    @property
+    def workspace_uuid(self) -> typing.Optional[uuid.UUID]:
+        uuid_str = typing.cast(typing.Optional[str], self._get_persistent_property_value("workspace_uuid"))
+        return uuid.UUID(uuid_str) if uuid_str else None
+
+    @workspace_uuid.setter
+    def workspace_uuid(self, value: typing.Optional[uuid.UUID]) -> None:
+        self._set_persistent_property_value("workspace_uuid", str(value) if value else None)
+
+    @property
+    def mapped_items(self) -> typing.List[Persistence._SpecifierType]:
+        return list(self._get_persistent_property_value("mapped_items"))
+
+    @mapped_items.setter
+    def mapped_items(self, value: typing.List[Persistence._SpecifierType]) -> None:
+        self._set_persistent_property_value("mapped_items", value)
+
+    @property
+    def data_items(self) -> typing.Sequence[DataItem.DataItem]:
+        return typing.cast(typing.Sequence[DataItem.DataItem], self._get_relationship_values("data_items"))
+
+    @property
+    def display_items(self) -> typing.Sequence[DisplayItem.DisplayItem]:
+        return typing.cast(typing.Sequence[DisplayItem.DisplayItem], self._get_relationship_values("display_items"))
+
+    @property
+    def computations(self) -> typing.Sequence[Symbolic.Computation]:
+        return typing.cast(typing.Sequence[Symbolic.Computation], self._get_relationship_values("computations"))
+
+    @property
+    def data_structures(self) -> typing.Sequence[DataStructure.DataStructure]:
+        return typing.cast(typing.Sequence[DataStructure.DataStructure], self._get_relationship_values("data_structures"))
+
+    @property
+    def connections(self) -> typing.Sequence[Connection.Connection]:
+        return typing.cast(typing.Sequence[Connection.Connection], self._get_relationship_values("connections"))
+
+    @property
+    def data_groups(self) -> typing.Sequence[DataGroup.DataGroup]:
+        return typing.cast(typing.Sequence[DataGroup.DataGroup], self._get_relationship_values("data_groups"))
+
+    @property
+    def workspaces(self) -> typing.Sequence[WorkspaceLayout.WorkspaceLayout]:
+        return typing.cast(typing.Sequence[WorkspaceLayout.WorkspaceLayout], self._get_relationship_values("workspaces"))
+
     def open(self) -> None:
         self.__storage_system.reset()  # this makes storage reusable during tests
 
-    def create_proxy(self) -> Persistence.PersistentObjectProxy:
-        return self.container.create_item_proxy(item=self)
+    def create_proxy(self) -> Persistence.PersistentObjectProxy[Project]:
+        container = self.container
+        assert container
+        return container.create_item_proxy(item=self)
 
     @property
     def item_specifier(self) -> Persistence.PersistentObjectSpecifier:
-        return Persistence.PersistentObjectSpecifier(item_uuid=self.uuid)
+        return Persistence.PersistentObjectSpecifier(self.uuid)
 
-    def create_specifier(self, item: Persistence.PersistentObject) -> Persistence.PersistentObjectSpecifier:
-        return Persistence.PersistentObjectSpecifier(item=item)
-
-    def insert_model_item(self, container, name, before_index, item) -> None:
+    def insert_model_item(self, container: Persistence.PersistentContainerType, name: str, before_index: int, item: Persistence.PersistentObject) -> None:
         # special handling to pass on to the document model
         assert callable(self.handle_insert_model_item)
         self.handle_insert_model_item(container, name, before_index, item)
 
-    def remove_model_item(self, container, name, item, *, safe: bool=False) -> Changes.UndeleteLog:
+    def remove_model_item(self, container: Persistence.PersistentContainerType, name: str, item: Persistence.PersistentObject, *, safe: bool = False) -> Changes.UndeleteLog:
         # special handling to pass on to the document model
         assert callable(self.handle_remove_model_item)
-        return self.handle_remove_model_item(container, name, item, safe=safe)
+        return self.handle_remove_model_item(container, name, item, safe)
 
     @property
     def storage_system_path(self) -> pathlib.Path:
         return pathlib.Path(self.__storage_system.get_identifier())
 
+    @property
+    def storage_cache(self) -> Cache.CacheLike:
+        assert self.__cache
+        return self.__cache
+
     @property
     def project_uuid(self) -> typing.Optional[uuid.UUID]:
         properties = self.__storage_system.get_storage_properties()
@@ -133,8 +201,9 @@ class Project(Observable.Observable, Persistence.PersistentObject):
     @property
     def project_filter(self) -> ListModel.Filter:
 
-        def is_display_item_active(project_weak_ref, display_item: DisplayItem.DisplayItem) -> bool:
-            return display_item.project == project_weak_ref()
+        # Python 3.9+ weak ref
+        def is_display_item_active(project_weak_ref: typing.Any, display_item: DisplayItem.DisplayItem) -> bool:
+            return bool(display_item.project == project_weak_ref())
 
         # use a weak reference to avoid circular references loops that prevent garbage collection
         return ListModel.PredicateFilter(functools.partial(is_display_item_active, weakref.ref(self)))
@@ -179,13 +248,13 @@ class Project(Observable.Observable, Persistence.PersistentObject):
     def __data_group_removed(self, name: str, index: int, data_group: DataGroup.DataGroup) -> None:
         self.notify_remove_item("data_groups", data_group, index)
 
-    def _get_relationship_persistent_dict(self, item, key: str, index: int) -> typing.Dict:
+    def _get_relationship_persistent_dict(self, item: Persistence.PersistentObject, key: str, index: int) -> typing.Optional[PersistentDictType]:
         if key == "data_items":
             return self.__storage_system.get_persistent_dict("data_items", item.uuid)
         else:
             return super()._get_relationship_persistent_dict(item, key, index)
 
-    def _get_relationship_persistent_dict_by_uuid(self, item, key: str) -> typing.Optional[typing.Dict]:
+    def _get_relationship_persistent_dict_by_uuid(self, item: Persistence.PersistentObject, key: str) -> typing.Optional[PersistentDictType]:
         if key == "data_items":
             return self.__storage_system.get_persistent_dict("data_items", item.uuid)
         else:
@@ -193,9 +262,19 @@ class Project(Observable.Observable, Persistence.PersistentObject):
 
     def prepare_read_project(self) -> None:
         logging.getLogger("loader").info(f"Loading project {self.__storage_system.get_identifier()}")
-        self._raw_properties = self.__storage_system.read_project_properties()  # combines library and data item properties
+        self._raw_properties, self.__reader_errors = self.__storage_system.read_project_properties()  # combines library and data item properties
         self.uuid = uuid.UUID(self._raw_properties.get("uuid", str(uuid.uuid4())))
 
+        for reader_error in self.__reader_errors:
+            logging.getLogger("loader").error(f"Error reading {reader_error.identifier} ({reader_error.exception})")
+            if False and reader_error.tb is not None:
+                import sys
+                import traceback
+                print(f"Event Listener Traceback (most recent call last)", file=sys.stderr)
+                frame_summaries = typing.cast(typing.List[typing.Any], reader_error.tb)
+                for line in traceback.StackSummary.from_list(frame_summaries).format():
+                    print(line, file=sys.stderr, end="")
+
     def read_project(self) -> None:
         if callable(self.handle_start_read):
             self.handle_start_read()
@@ -244,22 +323,24 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                         computation.close()
                 for item_d in properties.get("connections", list()):
                     connection = Connection.connection_factory(item_d.get)
-                    connection.begin_reading()
-                    connection.read_from_dict(item_d)
-                    connection.finish_reading()
-                    if not self.get_item_by_uuid("connections", connection.uuid):
-                        self.load_item("connections", len(self.connections), connection)
-                    else:
-                        connection.close()
+                    if connection:
+                        connection.begin_reading()
+                        connection.read_from_dict(item_d)
+                        connection.finish_reading()
+                        if not self.get_item_by_uuid("connections", connection.uuid):
+                            self.load_item("connections", len(self.connections), connection)
+                        else:
+                            connection.close()
                 for item_d in properties.get("data_groups", list()):
                     data_group = DataGroup.data_group_factory(item_d.get)
-                    data_group.begin_reading()
-                    data_group.read_from_dict(item_d)
-                    data_group.finish_reading()
-                    if not self.get_item_by_uuid("data_groups", data_group.uuid):
-                        self.load_item("data_groups", len(self.data_groups), data_group)
-                    else:
-                        data_group.close()
+                    if data_group:
+                        data_group.begin_reading()
+                        data_group.read_from_dict(item_d)
+                        data_group.finish_reading()
+                        if not self.get_item_by_uuid("data_groups", data_group.uuid):
+                            self.load_item("data_groups", len(self.data_groups), data_group)
+                        else:
+                            data_group.close()
                 for item_d in properties.get("workspaces", list()):
                     workspace = WorkspaceLayout.factory(item_d.get)
                     workspace.begin_reading()
@@ -270,15 +351,17 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                     else:
                         workspace.close()
                 workspace_uuid_str = properties.get("workspace_uuid", None)
-                if workspace_uuid_str:
-                    self._set_persistent_property_value("workspace_uuid", uuid.UUID(workspace_uuid_str))
-                self._set_persistent_property_value("data_item_references", properties.get("data_item_references", dict()))
-                self._set_persistent_property_value("mapped_items", properties.get("mapped_items", list()))
+                workspace_uuid = uuid.UUID(workspace_uuid_str) if workspace_uuid_str else None
+                existing_workspace_uuid = self._get_persistent_property_value("workspace_uuid", None)
+                if workspace_uuid and existing_workspace_uuid != workspace_uuid:
+                    self._get_persistent_property("workspace_uuid").set_value(str(workspace_uuid))
+                self._get_persistent_property("data_item_references").set_value(properties.get("data_item_references", dict()))
+                self._get_persistent_property("mapped_items").set_value(properties.get("mapped_items", list()))
                 self.__has_been_read = True
         if callable(self.handle_finish_read):
             self.handle_finish_read()
 
-    def __property_changed(self, name, value):
+    def __property_changed(self, name: str, value: typing.Any) -> None:
         self.notify_property_changed(name)
 
     def append_data_item(self, data_item: DataItem.DataItem) -> None:
@@ -333,12 +416,12 @@ class Project(Observable.Observable, Persistence.PersistentObject):
         self.remove_item("connections", connection)
 
     @property
-    def data_item_references(self) -> typing.Dict[str, uuid.UUID]:
+    def data_item_references(self) -> typing.Dict[str, str]:
         return dict(self._get_persistent_property_value("data_item_references").items())
 
     def set_data_item_reference(self, key: str, data_item: DataItem.DataItem) -> None:
         data_item_references = self.data_item_references
-        data_item_references[key] = data_item.item_specifier.write()
+        data_item_references[key] = str(data_item.item_specifier.write())
         self._set_persistent_property_value("data_item_references", {k: v for k, v in data_item_references.items()})
 
     def clear_data_item_reference(self, key: str) -> None:
@@ -346,14 +429,6 @@ class Project(Observable.Observable, Persistence.PersistentObject):
         del data_item_references[key]
         self._set_persistent_property_value("data_item_references", {k: v for k, v in data_item_references.items()})
 
-    @property
-    def mapped_items(self) -> typing.List[typing.Union[typing.Mapping, str]]:
-        return list(self._get_persistent_property_value("mapped_items"))
-
-    @mapped_items.setter
-    def mapped_items(self, value: typing.List[typing.Union[typing.Mapping, str]]) -> None:
-        self._set_persistent_property_value("mapped_items", value)
-
     def prune(self) -> None:
         self.__storage_system.prune()
 
@@ -379,20 +454,21 @@ class Project(Observable.Observable, Persistence.PersistentObject):
             self.unload_item("data_items", len(self.data_items) - 1)
 
 
-def data_item_factory(lookup_id):
+def data_item_factory(lookup_id: typing.Callable[[str], str]) -> DataItem.DataItem:
     data_item_uuid = uuid.UUID(lookup_id("uuid"))
-    large_format = lookup_id("__large_format", False)
+    # TODO: typing hack for default arg
+    large_format = typing.cast(typing.Callable[..., typing.Any], lookup_id)("__large_format", False)
     return DataItem.DataItem(item_uuid=data_item_uuid, large_format=large_format)
 
 
-def display_item_factory(lookup_id):
+def display_item_factory(lookup_id: typing.Callable[[str], str]) -> DisplayItem.DisplayItem:
     display_item_uuid = uuid.UUID(lookup_id("uuid"))
     return DisplayItem.DisplayItem(item_uuid=display_item_uuid)
 
 
-def computation_factory(lookup_id):
+def computation_factory(lookup_id: typing.Callable[[str], str]) -> Symbolic.Computation:
     return Symbolic.Computation()
 
 
-def data_structure_factory(lookup_id):
+def data_structure_factory(lookup_id: typing.Callable[[str], str]) -> DataStructure.DataStructure:
     return DataStructure.DataStructure()
