@@ -1,5 +1,6 @@
 # standard libraries
 import logging
+import math
 import unittest
 
 # third party libraries
@@ -12,6 +13,7 @@ from nion.swift import Application
 from nion.swift.model import DataItem
 from nion.swift.model import Graphics
 from nion.swift.test import TestContext
+from nion.ui import DrawingContext
 from nion.ui import TestUI
 
 
@@ -74,7 +76,7 @@ class TestImageCanvasItemClass(unittest.TestCase):
             header_height = display_panel.header_canvas_item.header_height
             display_panel.root_container.layout_immediate((1000 + header_height, 1000))
             # run test
-            self.assertEqual(display_panel.display_canvas_item._info_overlay_canvas_item_for_test._dimension_calibration_for_test.units, "x")
+            self.assertEqual(display_panel.display_canvas_item._scale_marker_canvas_item_for_test._dimension_calibration_for_test.units, "x")
 
     def test_dimension_used_for_scale_marker_on_3d_spectrum_image_is_correct(self):
         with TestContext.create_memory_context() as test_context:
@@ -90,7 +92,7 @@ class TestImageCanvasItemClass(unittest.TestCase):
             header_height = display_panel.header_canvas_item.header_height
             display_panel.root_container.layout_immediate((1000 + header_height, 1000))
             # run test
-            self.assertEqual(display_panel.display_canvas_item._info_overlay_canvas_item_for_test._dimension_calibration_for_test.units, "x")
+            self.assertEqual(display_panel.display_canvas_item._scale_marker_canvas_item_for_test._dimension_calibration_for_test.units, "x")
 
     def test_dimension_used_for_scale_marker_on_4d_diffraction_image_is_correct(self):
         with TestContext.create_memory_context() as test_context:
@@ -106,7 +108,21 @@ class TestImageCanvasItemClass(unittest.TestCase):
             header_height = display_panel.header_canvas_item.header_height
             display_panel.root_container.layout_immediate((1000 + header_height, 1000))
             # run test
-            self.assertEqual(display_panel.display_canvas_item._info_overlay_canvas_item_for_test._dimension_calibration_for_test.units, "b")
+            self.assertEqual(display_panel.display_canvas_item._scale_marker_canvas_item_for_test._dimension_calibration_for_test.units, "b")
+
+    def test_scale_marker_with_invalid_calibration(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            display_panel = document_controller.selected_display_panel
+            calibrations = [Calibration.Calibration(scale=math.nan), Calibration.Calibration(scale=math.nan)]
+            data_and_metadata = DataAndMetadata.new_data_and_metadata(numpy.ones((10, 10)), dimensional_calibrations=calibrations)
+            data_item = DataItem.new_data_item(data_and_metadata)
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_panel.set_display_panel_display_item(display_item)
+            header_height = display_panel.header_canvas_item.header_height
+            display_panel.root_container.layout_immediate((1000 + header_height, 1000))
 
     def test_tool_returns_to_pointer_after_but_not_during_creating_rectangle(self):
         # setup
@@ -247,6 +263,8 @@ class TestImageCanvasItemClass(unittest.TestCase):
             display_item.display_type = "image"
             header_height = display_panel.header_canvas_item.header_height
             display_panel.root_container.layout_immediate((1000 + header_height, 1000))
+            drawing_context = DrawingContext.DrawingContext()
+            display_panel.root_container.repaint_immediate(drawing_context, display_panel.root_container.canvas_size)
 
     def test_hand_tool_on_one_image_of_multiple_displays(self):
         # setup
