@@ -1,7 +1,8 @@
 # standard libraries
-import contextlib
 import logging
 import math
+import operator
+import typing
 import unittest
 
 # third party libraries
@@ -12,14 +13,46 @@ from nion.data import Calibration
 from nion.data import DataAndMetadata
 from nion.swift import Application
 from nion.swift import LineGraphCanvasItem
+from nion.swift import LinePlotCanvasItem
 from nion.swift.model import DataItem
 from nion.swift.model import DisplayItem
 from nion.swift.model import Graphics
 from nion.swift.test import TestContext
+from nion.ui import CanvasItem
 from nion.ui import DrawingContext
 from nion.ui import TestUI
+from nion.utils import Color
+from nion.utils import Geometry
 
 
+def _enumerate_child_canvas_items_postorder(
+        canvas_item: CanvasItem.AbstractCanvasItem,
+        depth: int
+) -> typing.Iterator[typing.Tuple[CanvasItem.AbstractCanvasItem, int]]:
+    if isinstance(canvas_item, CanvasItem.CanvasItemComposition):
+        for child_canvas_item in canvas_item.canvas_items:
+            yield from _enumerate_child_canvas_items_postorder(child_canvas_item, depth+1)
+
+    yield canvas_item, depth
+
+
+def _find_first_descendent_of_type_postorder(
+        canvas_item: CanvasItem.AbstractCanvasItem,
+        canvas_item_type
+) -> typing.Optional[CanvasItem.AbstractCanvasItem]:
+    return next(
+        filter(
+            (lambda _: isinstance(_, canvas_item_type)),
+            map(operator.itemgetter(0), _enumerate_child_canvas_items_postorder(canvas_item, 0))
+        ),
+        None
+    )
+
+def _print_canvas_item_tree_preorder(canvas_item: CanvasItem.AbstractCanvasItem):
+
+    for ci, depth in _enumerate_child_canvas_items_postorder(canvas_item, 0):
+        print("0x{:016x} - {} {} - {}".format(id(ci), depth * '   ', type(ci).__name__, ci.canvas_bounds))
+
 def create_memory_profile_context() -> TestContext.MemoryProfileContext:
     return TestContext.MemoryProfileContext()
 
@@ -50,7 +83,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             data[:] = data_min + (data_max - data_min) * (irow / 15.0)
             # auto on min/max
             calibrated_data_min, calibrated_data_max, y_ticker = LineGraphCanvasItem.calculate_y_axis([data], None, None, None, None)
-            axes = LineGraphCanvasItem.LineGraphAxes(1.0, calibrated_data_min, calibrated_data_max, y_ticker=y_ticker)
+            axes = LineGraphCanvasItem.LineGraphAxes(1.0, calibrated_data_min, calibrated_data_max, 0, 100, None, None, None, y_ticker)
             self.assertEqual(axes.uncalibrated_data_min, expected_uncalibrated_data_min)
             self.assertEqual(axes.uncalibrated_data_max, expected_uncalibrated_data_max)
 
@@ -70,10 +103,11 @@ class TestLineGraphCanvasItem(unittest.TestCase):
                 data = numpy.linspace(data_in[0], data_in[1], 100, endpoint=False)
                 data_style = "log"
                 calibrated_data_min, calibrated_data_max, y_ticker = LineGraphCanvasItem.calculate_y_axis([data], None, None, None, data_style)
-                axes = LineGraphCanvasItem.LineGraphAxes(1.0, calibrated_data_min, calibrated_data_max, data_style=data_style, y_ticker=y_ticker)
+                axes = LineGraphCanvasItem.LineGraphAxes(1.0, calibrated_data_min, calibrated_data_max, 0, 100, None, None, data_style, y_ticker)
                 self.assertAlmostEqual(axes.uncalibrated_data_min, data_out[0], places=1)
                 self.assertAlmostEqual(axes.uncalibrated_data_max, data_out[1], places=1)
                 calibrated_data = axes.calculate_calibrated_xdata(DataAndMetadata.new_data_and_metadata(data)).data
+                assert calibrated_data is not None
                 data[data <= 0] = numpy.nan
                 self.assertAlmostEqual(numpy.nanmin(calibrated_data), math.log10(numpy.nanmin(data)))
                 self.assertAlmostEqual(numpy.nanmax(calibrated_data), math.log10(numpy.nanmax(data)))
@@ -95,10 +129,11 @@ class TestLineGraphCanvasItem(unittest.TestCase):
                 intensity_calibration = Calibration.Calibration(-5, 2)
                 data_style = "log"
                 calibrated_data_min, calibrated_data_max, y_ticker = LineGraphCanvasItem.calculate_y_axis([data], None, None, intensity_calibration, data_style)
-                axes = LineGraphCanvasItem.LineGraphAxes(1.0, calibrated_data_min, calibrated_data_max, data_style=data_style, y_ticker=y_ticker)
+                axes = LineGraphCanvasItem.LineGraphAxes(1.0, calibrated_data_min, calibrated_data_max, 0, 100, None, None, data_style, y_ticker)
                 self.assertAlmostEqual(axes.uncalibrated_data_min, data_out[0], places=1)
                 self.assertAlmostEqual(axes.uncalibrated_data_max, data_out[1], places=1)
                 calibrated_data = axes.calculate_calibrated_xdata(DataAndMetadata.new_data_and_metadata(data)).data
+                assert calibrated_data is not None
                 data[data <= 0] = numpy.nan
                 self.assertAlmostEqual(numpy.nanmin(calibrated_data), math.log10(numpy.nanmin(data)))
                 self.assertAlmostEqual(numpy.nanmax(calibrated_data), math.log10(numpy.nanmax(data)))
@@ -181,7 +216,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             # test
             document_controller.tool_mode = "pointer"
             display_panel.display_canvas_item.simulate_click((240, 320))
-            display_panel.display_canvas_item.key_pressed(self.app.ui.create_key_by_id("left"))
+            display_panel._handle_key_pressed(typing.cast(TestUI.UserInterface, self.app.ui).create_key_by_id("left"))
             interval_region = display_item.graphics[0]
             self.assertTrue(interval_region.start < 0.1)
             self.assertTrue(interval_region.end < 0.9)
@@ -238,10 +273,11 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_panel.layout_immediate((640, 480))
             axes = display_panel.display_canvas_item._axes
             drawing_context = DrawingContext.DrawingContext()
-            calibrated_data_min = axes.calibrated_data_min
-            calibrated_data_max = axes.calibrated_data_max
-            calibrated_data_range = calibrated_data_max - calibrated_data_min
-            LineGraphCanvasItem.draw_line_graph(drawing_context, 480, 640, 0, 0, data_item.xdata, calibrated_data_min, calibrated_data_range, axes.calibrated_left_channel, axes.calibrated_right_channel, axes.x_calibration, "black", "black", None, axes.data_style)
+            line_graph_layer = LineGraphCanvasItem.LineGraphLayer(data_item.xdata, Color.Color("black"), Color.Color("black"), None)
+            line_graph_layer.set_axes(axes)
+            line_graph_layer.calculate(Geometry.IntRect.from_tlbr(0, 0, 480, 640))
+            line_graph_layer.draw_fills(drawing_context)
+            line_graph_layer.draw_strokes(drawing_context)
             # ensure that the drawing commands are sufficiently populated to have drawn the graph
             self.assertGreater(len(drawing_context.commands), 100)
 
@@ -261,11 +297,11 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             axes = display_panel.display_canvas_item._axes
             self.assertEqual(axes.data_style, "log")
             drawing_context = DrawingContext.DrawingContext()
-            calibrated_data_min = axes.calibrated_data_min
-            calibrated_data_max = axes.calibrated_data_max
-            calibrated_data_range = calibrated_data_max - calibrated_data_min
-            display_xdata = display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata
-            LineGraphCanvasItem.draw_line_graph(drawing_context, 480, 640, 0, 0, display_xdata, calibrated_data_min, calibrated_data_range, axes.calibrated_left_channel, axes.calibrated_right_channel, axes.x_calibration, "black", "black", None, axes.data_style)
+            line_graph_layer = LineGraphCanvasItem.LineGraphLayer(data_item.xdata, Color.Color("black"), Color.Color("black"), None)
+            line_graph_layer.set_axes(axes)
+            line_graph_layer.calculate(Geometry.IntRect.from_tlbr(0, 0, 480, 640))
+            line_graph_layer.draw_fills(drawing_context)
+            line_graph_layer.draw_strokes(drawing_context)
             # ensure that the drawing commands are sufficiently populated to have drawn the graph
             self.assertGreater(len(drawing_context.commands), 100)
 
@@ -276,7 +312,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             data_item = DataItem.DataItem(numpy.ones((8,), float))
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
-            display_item.data_item.set_xdata(DataAndMetadata.DataAndMetadata(lambda: None, ((8, 0), float)))
+            display_item.data_item._force_unload()
             display_item.display_type = "line_plot"
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
@@ -293,10 +329,10 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
             display_panel.layout_immediate((640, 480))
-            self.assertTrue(numpy.array_equal(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.data, numpy.full((8, ), 10)))
+            self.assertTrue(numpy.array_equal(display_panel.display_canvas_item.line_graph_layers_canvas_item.calibrated_xdata.data, numpy.full((8, ), 10)))
             display_item.calibration_style_id = "pixels-top-left"
             display_panel.layout_immediate((640, 480))
-            self.assertTrue(numpy.array_equal(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.data, numpy.ones((8, ))))
+            self.assertTrue(numpy.array_equal(display_panel.display_canvas_item.line_graph_layers_canvas_item.calibrated_xdata.data, numpy.ones((8, ))))
 
     def test_line_plot_handles_calibrated_vs_uncalibrated_display(self):
         with TestContext.create_memory_context() as test_context:
@@ -309,10 +345,10 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
             display_panel.layout_immediate((640, 480))
-            self.assertEqual(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.dimensional_calibrations[-1].units, "nm")
+            self.assertEqual(display_panel.display_canvas_item.line_graph_layers_canvas_item.calibrated_xdata.dimensional_calibrations[-1].units, "nm")
             display_item.calibration_style_id = "pixels-top-left"
             display_panel.layout_immediate((640, 480))
-            self.assertFalse(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.dimensional_calibrations[-1].units)
+            self.assertFalse(display_panel.display_canvas_item.line_graph_layers_canvas_item.calibrated_xdata.dimensional_calibrations[-1].units)
 
     def test_line_plot_with_no_data_handles_clicks(self):
         with TestContext.create_memory_context() as test_context:
@@ -341,11 +377,11 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_panel.layout_immediate((640, 480))
             axes = display_panel.display_canvas_item._axes
             drawing_context = DrawingContext.DrawingContext()
-            calibrated_data_min = axes.calibrated_data_min
-            calibrated_data_max = axes.calibrated_data_max
-            calibrated_data_range = calibrated_data_max - calibrated_data_min
-            display_xdata = display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata
-            LineGraphCanvasItem.draw_line_graph(drawing_context, 480, 100, 0, 0, display_xdata, calibrated_data_min, calibrated_data_range, axes.calibrated_left_channel, axes.calibrated_right_channel, axes.x_calibration, "black", "black", None, axes.data_style)
+            line_graph_layer = LineGraphCanvasItem.LineGraphLayer(data_item.xdata, Color.Color("black"), Color.Color("black"), None)
+            line_graph_layer.set_axes(axes)
+            line_graph_layer.calculate(Geometry.IntRect.from_tlbr(0, 0, 480, 640))
+            line_graph_layer.draw_fills(drawing_context)
+            line_graph_layer.draw_strokes(drawing_context)
             # ensure that the drawing commands are sufficiently populated to have drawn the graph
             self.assertGreater(len(drawing_context.commands), 100)
 
@@ -410,6 +446,60 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_item1.display_type = None
             self.assertEqual(2, len(display_item1.display_layers))
 
+    def test_line_plot_width_doesnt_jitter_for_small_vertical_bounds_changes(self):
+        """
+        Create a line plot with data that has been shown, when dragged vertically, produces bounds with
+        a reasonable distribution of different digits. Verify that the layout calclated for these bounds
+        does not change for moderate sized drags.
+        """
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            display_panel = document_controller.selected_display_panel
+            data = numpy.sin(numpy.linspace(0, 20, 100)) * 53488.2
+            data_item = DataItem.DataItem(data)
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_panel.set_display_panel_display_item(display_item)
+
+            lp_ci = _find_first_descendent_of_type_postorder(
+                display_panel,
+                LinePlotCanvasItem.LinePlotCanvasItem)
+            lp_graph_frame_ci = _find_first_descendent_of_type_postorder(
+                display_panel,
+                LineGraphCanvasItem.LineGraphFrameCanvasItem
+            )
+            lp_vert_axis_scale_ci = _find_first_descendent_of_type_postorder(
+                display_panel,
+                LineGraphCanvasItem.LineGraphVerticalAxisScaleCanvasItem
+            )
+            display_panel.layout_immediate((960, 1200))
+
+            # _print_canvas_item_tree_preorder(display_panel)
+
+            # Drag vertical axis, and force re-layout every few pixels of drag.
+            # Record the distinct sizes calculate for line graph and vertical axis items.
+            lp_graph_frame_ci_sizes = set()
+            lp_vert_axis_scale_ci_sizes = set()
+
+            drag_start_pos_x = lp_vert_axis_scale_ci.canvas_size.width
+            drag_start_pos_y = lp_vert_axis_scale_ci.canvas_size.height // 2
+            for y_offset in range(0, 30):
+                lp_ci.simulate_drag(
+                    (drag_start_pos_y + 2*y_offset, drag_start_pos_x),
+                    (drag_start_pos_y + 2*(y_offset+1), drag_start_pos_x))
+
+                display_panel.refresh_layout_immediate()
+
+                lp_graph_frame_ci_sizes.add((lp_graph_frame_ci.canvas_size.width, lp_graph_frame_ci.canvas_size.height))
+                lp_vert_axis_scale_ci_sizes.add((lp_vert_axis_scale_ci.canvas_size.width, lp_vert_axis_scale_ci.canvas_size.height))
+
+            # Verify that there is only one size that the line plot grapn and vertical axis scale
+            # items take during the drag, indicating that the layout is stable against small changes
+            # in bounds.
+            self.assertEqual(len(lp_graph_frame_ci_sizes), 1)
+            self.assertEqual(len(lp_vert_axis_scale_ci_sizes), 1)
+
     def test_check_exponents(self):
         e = LineGraphCanvasItem.Exponenter()
         e.add_label("5e+05")
