@@ -478,8 +478,8 @@ class TestProcessingClass(unittest.TestCase):
             document_model.append_data_item(rgba_data_item)
             rgba_display_item = document_model.get_display_item_for_data_item(rgba_data_item)
             with rgba_display_item.data_item.data_ref() as data_ref:
-                data_ref.master_data[:] = (20,40,60,100)
-                data_ref.master_data_updated()
+                data_ref.data[:] = (20,40,60,100)
+                data_ref.data_updated()
             rgba2_data_item = document_model.get_invert_new(rgba_display_item, rgba_display_item.data_item)
             rgba2_display_item = document_model.get_display_item_for_data_item(rgba2_data_item)
             document_model.recompute_all()
@@ -520,7 +520,6 @@ class TestProcessingClass(unittest.TestCase):
         with TestContext.create_memory_context() as test_context:
             document_model = test_context.create_document_model()
             data_item = DataItem.DataItem()
-            data_item.ensure_data_source()
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             self.assertIsNone(display_item.data_item.data)
@@ -1018,7 +1017,7 @@ class TestProcessingClass(unittest.TestCase):
             document_model.recompute_all()
             redim_data_item = document_model.data_items[1]
             self.assertEqual(redim_data_item.xdata.data_descriptor, DataAndMetadata.DataDescriptor(False, 1, 2))
-            self.assertEqual(redim_data_item.xdata.dimensional_calibrations, [Calibration.Calibration(1), Calibration.Calibration(2), Calibration.Calibration(3)])
+            self.assertEqual(list(redim_data_item.xdata.dimensional_calibrations), [Calibration.Calibration(1), Calibration.Calibration(2), Calibration.Calibration(3)])
 
     def test_squeeze_removes_proper_dimensions(self):
         with TestContext.create_memory_context() as test_context:
@@ -1040,7 +1039,7 @@ class TestProcessingClass(unittest.TestCase):
             document_model.recompute_all()
             squeezed_data_item = document_model.data_items[1]
             self.assertEqual(squeezed_data_item.xdata.data_descriptor, DataAndMetadata.DataDescriptor(False, 1, 1))
-            self.assertEqual(squeezed_data_item.xdata.dimensional_calibrations, [Calibration.Calibration(2), Calibration.Calibration(4)])
+            self.assertEqual(list(squeezed_data_item.xdata.dimensional_calibrations), [Calibration.Calibration(2), Calibration.Calibration(4)])
 
     def test_squeeze_removes_proper_dimensions_when_both_datum_is_1(self):
         with TestContext.create_memory_context() as test_context:
@@ -1063,7 +1062,7 @@ class TestProcessingClass(unittest.TestCase):
             document_model.recompute_all()
             squeezed_data_item = document_model.data_items[1]
             self.assertEqual(squeezed_data_item.xdata.data_descriptor, DataAndMetadata.DataDescriptor(False, 1, 1))
-            self.assertEqual(squeezed_data_item.xdata.dimensional_calibrations, [Calibration.Calibration(2), Calibration.Calibration(5)])
+            self.assertEqual(list(squeezed_data_item.xdata.dimensional_calibrations), [Calibration.Calibration(2), Calibration.Calibration(5)])
 
     def test_invalid_processing_produces_no_output(self):
         with TestContext.create_memory_context() as test_context:
